import QtQuick 2.12
import QtQuick.Window 2.12
import QtQuick.Controls 2.12
import MuseScore 3.0

MuseScore {
    menuPath: "D:/Documents/MuseScore3/NoteNumber"
    description: "Note namber"
    version: "3.0"
    pluginType: "dialog"
    requiresScore: false
//Window {
    id: window
    width: 310
    height: 480
    visible: true
    //title: qsTr("Note info")
    property var keys: [0,1,0,1,0,0,1,0,1,0,1,0]
    property var altSign: ["♭","♯"]
    Component.onCompleted: { start() }

    Row {
        id: rowTop
        anchors.horizontalCenter: parent.horizontalCenter
        spacing: 10
        padding: 10
        ComboBox {
            id: comboBoxKey
            model: getModelKeys()
        }

        ComboBox {
            id: comboBoxHarmony
            model: ["Major", "Minor"]
        }
    }

    ListView {
        id: listView
        anchors.fill: parent
        anchors.bottomMargin: 60
        anchors.topMargin: 60

        headerPositioning: ListView.OverlayHeader
        header: Rectangle {
            id: headerItem
            width: listView.width
            height:30
            Row {
                spacing: 5
                padding: 5
                Repeater{
                    model: ["#", "name", "oktave", "pitch"]
                    Rectangle{
                        width: (window.width - 25) / 4
                        height:30
                        color: "#e0e0e0"
                        Text {
                            anchors.fill: parent
                            horizontalAlignment: Text.AlignHCenter
                            verticalAlignment: Text.AlignVCenter
                            font.capitalization: Font.AllUppercase
                            text: modelData
                        }
                    }
                }
            }
        }
        delegate: Item {
            width: window.width
            height: 30
            Rectangle{
                anchors.fill: parent
                color: "Ivory"
                Row {
                    id: row1
                    spacing: 5
                    padding: 5
                    Repeater{
                        model: listView.model[index]
                        Rectangle {
                            width: (window.width - 25) / 4
                            height: 30
                            Text {
                                anchors.fill: parent
                                horizontalAlignment: Text.AlignHCenter
                                verticalAlignment: Text.AlignVCenter
                                text: modelData
                            }
                        }
                    }
                }
            }
        }
    }

    Row {
        id: rowBottom
        spacing: 10
        padding: 10
        anchors.bottom: parent.bottom
        anchors.horizontalCenter: parent.horizontalCenter

        Button {
            id: buttonStart
            text: qsTr("Start")
            onClicked: { setNoteText(); }
        }
    }


    function getNextItem (value, step, limit, size) {
        var result = value + step;
        if (result > limit) { result -= size; }
        if (result < (limit - size)) { result += size; }
        return result;
    }

    function getKeyIndex(pitch) {
        var keyIndex = pitch;
        while (keyIndex > 11) { keyIndex -= 12 }
        return keyIndex;
    }

    function getNumber(pitch) {
        var keyIndex = getKeyIndex(pitch);
        var alteration = 0;
        var countWhiteKeys = 0;
        for (var i = 0; i <= keyIndex; i++) {
            alteration = keys[i];
            if (alteration === 0) { countWhiteKeys++}
        }

        var result = getNextItem (countWhiteKeys, -comboBoxKey.currentIndex, 7, 7);
        if (alteration > 0) {
            countWhiteKeys++;
            var nextItem = getNextItem (countWhiteKeys, 1, 7, 7);
            result += altSign[1] + "/" + nextItem + altSign[0];
        }
        return "" + result;
    }

    function getOktave(pitch) {
        var result = 0;
        var i = -1;
        while(i < pitch) {
            i += 12;
            result++;
        }
        return result;
    }

    function getLetter(pitch) {
        var keyIndex = getKeyIndex(pitch);
        var alteration = 0;
        var countWhiteKeys = -1;
        for (var i = 0; i <= keyIndex; i++) {
            alteration = keys[i];
            if (alteration === 0) { countWhiteKeys++}
        }
        var charCode = getNextItem (67, countWhiteKeys, 71, 7);
        var result = String.fromCharCode(charCode);
        if (alteration > 0) {
            charCode = getNextItem (charCode, 1, 71, 7);
            result += altSign[1] + "/" + String.fromCharCode(charCode) + altSign[0];
        }
        return result;
    }

    function start() {
        comboBoxKey.currentIndex = 2;

        setListModel();
    }

    function getNumberArr(pitch) {
        return [getNumber(pitch), getLetter(pitch), getOktave(pitch), "" + pitch];
    }


    function setListModel() {
        var modelArr = [];
        for (var i = 0; i < 26; i++) {
            modelArr[i] = getNumberArr(i + 48);
        }
        listView.model = modelArr;
    }

    function getModelKeys() {
        var modelArr = [];
        for (var i = 0; i < 12; i++) {
            modelArr[i] = getLetter(i);
        }
        comboBoxKey.model = modelArr;
    }

    function getNoteText(notes, listViewModel) {
        var noteText = "";
        for (var i = 0; i < notes.length; i++) {
            var note = notes[i];
            if ( i > 0 )
                noteText  = "\n" + noteText;
            if (typeof note.pitch === "undefined")
                continue

            var itemModel = getNumberArr(note.pitch);
            listViewModel.push(getNumberArr(note.pitch));
            noteText  = itemModel[0] + noteText;

        }

        var textHord = newElement(Element.STAFF_TEXT);
        textHord.text = noteText;
        return textHord;
    }


    function setNoteText() {
        var listViewModel = [];
        var noteText;
        var cursor = curScore.newCursor();
        for (var staff = 0; staff < curScore.nstaves; staff++) {
            for (var voice = 0; voice < 4; voice++) {
                cursor.rewind(0);
                cursor.voice = voice;
                cursor.staffIdx = staff;
                while (cursor.element) {
                    if (cursor.element.type === Element.CHORD) {
                        for (var i = 0; i < cursor.element.graceNotes.length; i++) {
                            noteText = getNoteText(cursor.element.graceNotes[i].notes, listViewModel)
                            cursor.add(noteText);
                        }
                        noteText = getNoteText(cursor.element.notes, listViewModel)
                        cursor.add(noteText);
                    }
                    cursor.next();
                }
            }
        }
        listView.model = listViewModel;
    }


}
